@@ -283,8 +283,9 @@ public:
     * @return temporary filename
     */
   svn::Path
-  Action::GetPathAsTempFile (const svn::Path & path, 
-            const svn::Revision & revision = svn::Revision::HEAD);
+  Action::GetPathAsTempFile (
+    const svn::Path & path, 
+    const svn::Revision & revision = svn::Revision::HEAD);
 
 protected:
 
