@@ -34,7 +34,6 @@
 #include "ids.hpp"
 #include "filelist_ctrl.hpp"
 #include "utils.hpp"
-#include "verblist.hpp"
 #include "preferences.hpp"
 #include "action.hpp"
 #include "rapidsvn_app.hpp"
@@ -198,20 +197,20 @@ Compare (svn_wc_status_kind status1, svn_wc_status_kind status2, bool newer1, bo
   
   static svn_wc_status_kind lookup[] = 
   {
-    svn_wc_status_deleted,
-    svn_wc_status_conflicted,
-    svn_wc_status_added,
-    FAKE_SVN_WC_OUT_OF_DATE,
-    svn_wc_status_missing,
-    svn_wc_status_replaced,
-    svn_wc_status_merged,
-    svn_wc_status_modified,
-    svn_wc_status_normal,
     svn_wc_status_external,
     svn_wc_status_ignored,
     svn_wc_status_incomplete,
     svn_wc_status_obstructed,
     svn_wc_status_unversioned,
+    svn_wc_status_deleted,
+    svn_wc_status_conflicted,
+    svn_wc_status_added,
+    svn_wc_status_missing,
+    svn_wc_status_replaced,
+    svn_wc_status_merged,
+    svn_wc_status_modified,
+    FAKE_SVN_WC_OUT_OF_DATE,
+    svn_wc_status_normal,
     svn_wc_status_none,
     (svn_wc_status_kind)0
   };
@@ -736,10 +735,10 @@ GetImageIndex (int textIndex, int propIndex)
 BEGIN_EVENT_TABLE (FileListCtrl, wxListCtrl)
   EVT_KEY_DOWN (FileListCtrl::OnKeyDown)
   EVT_LEFT_DCLICK (FileListCtrl::OnDoubleClick)
-  EVT_LIST_ITEM_ACTIVATED (-1, FileListCtrl::OnItemActivated)
-  EVT_LIST_ITEM_RIGHT_CLICK (FILELIST_CTRL, FileListCtrl::OnItemRightClk)
   EVT_LIST_COL_CLICK (FILELIST_CTRL, FileListCtrl::OnColumnLeftClick)
   EVT_LIST_COL_END_DRAG (FILELIST_CTRL, FileListCtrl::OnColumnEndDrag)
+  EVT_LIST_ITEM_RIGHT_CLICK (FILELIST_CTRL, FileListCtrl::OnItemRightClk)
+  EVT_CONTEXT_MENU (FileListCtrl::OnContextMenu)
 END_EVENT_TABLE ()
 
 FileListCtrl::FileListCtrl (wxWindow * parent, const wxWindowID id, 
@@ -1030,20 +1029,45 @@ FileListCtrl::OnKeyDown (wxKeyEvent & event)
 }
 
 void
-FileListCtrl::OnItemActivated (wxListEvent & event)
+FileListCtrl::OnItemRightClk (wxListEvent & event)
 {
-  //Unused now
+  RapidSvnFrame* frame = (RapidSvnFrame*) wxGetApp ().GetTopWindow ();
+  frame->SetActivePane (ACTIVEPANE_FILELIST);
+
+#ifdef __WXGTK__
+  // wxGTK doesn't seem to correctly select an item upon right clicking - try doing it ourselves for now
+  wxPoint clientPt = event.GetPoint ();
+  int flags;
+  long id = HitTest (clientPt, flags);
+  if (id >= 0)
+  {
+    if (!IsSelected (id))
+    {
+      // Need to unselect all currently selected
+      long sel = GetFirstSelected ();
+      while (sel != -1)
+      {
+        Select (sel, false);
+        sel = GetNextSelected (sel);
+      }
+      // Finally select the one just clicked
+      Select (id);
+    }
+  }
+  // Show the menu now rather than waiting for OnContextMenu,
+  // and so don't Skip ().
+  ShowMenu (clientPt);
+#else
+  // Let the OnContextMenu handler do the menu on MouseUp
+  event.Skip ();
+#endif
 }
 
 void
-FileListCtrl::OnItemRightClk (wxListEvent & event)
+FileListCtrl::OnContextMenu (wxContextMenuEvent & event)
 {
-  wxPoint pt (event.GetPoint ());
-  int flags;
-  bool test = HitTest (pt, flags) >= 0;
-
-  if ((GetSelectedItemCount () > 0) && test)
-    ShowMenu (pt);
+  wxPoint clientPt (ScreenToClient (event.GetPosition ()));
+  ShowMenu (clientPt);
 }
 
 void
@@ -1079,60 +1103,8 @@ FileListCtrl::ShowMenu (wxPoint & pt)
     long item = GetNextItem (-1, wxLIST_NEXT_ALL, 
                              wxLIST_STATE_SELECTED);
     svn::Status * status = (svn::Status*)GetItemData (item);
-    VerbList verbList;
 
-    // Append file verbs
-    try
-    {
-      if(status != 0)
-      verbList.InitFromDocument (status->path ());
-    }
-    catch (std::exception)
-    {
-      // Failed assembling verbs. 
-      // TODO: Report this error in the status bar?
-    }
-
-    size_t i = 0;
-    for (; 
-         (i < verbList.GetCount ()) && 
-         (i < (ID_Verb_Max - ID_Verb_Min + 1)); i++)
-    {
-      wxMenuItem *pItem;
-      // TODO: Convert verb names to unicode on the fly if needed (or make
-      // verblist follow wxWindows' unicode setting)
-      pItem = new wxMenuItem (&menu, ID_Verb_Min + i, verbList.GetName (i));
-      //pItem->SetBitmap (wxBITMAP (?))
-      menu.Append (pItem);
-    }
-    
-    if (i == 0)
-    {
-      // No verbs - allow configured editor if there is one
-      Preferences prefs;
-      if (!prefs.editor.IsEmpty () || IsDir (status))
-      {
-        const char * defaultActionVerb = _("Open");
-        if (!IsDir (status))
-        {
-          if (svn::Url::isValid (status->path ()))
-          {
-            defaultActionVerb = _("View HEAD");
-          }
-          else
-          {
-            defaultActionVerb = _("Edit");
-          }
-        }
-        menu.Append (new wxMenuItem (&menu, ID_Default_Action, defaultActionVerb));
-        ++i;
-      }
-    }
-
-    if (i > 0)
-    {
-      menu.AppendSeparator ();
-    }
+    AppendVerbMenu (&menu, status);
   }
 
   AppendModifyMenu (&menu);
