@@ -144,6 +144,7 @@ public:
   wxTreeItemId rootId;
   wxImageList* imageList;
   Bookmarks bookmarks;
+  svn::Context defaultContext;
   
   Data (wxWindow * window, const wxPoint & pos, const wxSize & size)
     : window (window), listener (0)
@@ -160,7 +161,7 @@ public:
     imageList->Add (wxIcon (bookmark_xpm));
 
     treeCtrl = new wxTreeCtrl (window, -1, pos, size, 
-                               wxTR_HAS_BUTTONS);
+                               wxTR_HAS_BUTTONS|wxTR_SINGLE);
     treeCtrl->AssignImageList (imageList);
 
     FolderItemData* data = new FolderItemData (FOLDER_TYPE_BOOKMARKS);
@@ -208,36 +209,38 @@ public:
   GetContext () 
   {
     const FolderItemData * data = GetSelection ();
-    if (data == 0)
-      return 0;
-
     svn::Context * context = 0;
 
-    bool ok = true;
-    while (context == 0)
+    if (data == 0)
+      context = &defaultContext;
+    else
     {
-      switch (data->getFolderType ())
+      bool ok = true;
+      while (context == 0)
       {
-      case FOLDER_TYPE_NORMAL:
+        switch (data->getFolderType ())
         {
-          wxTreeItemId id = data->GetId ();
-          wxTreeItemId parentId = treeCtrl->GetItemParent (id);
-          data = GetItemData (parentId);
+        case FOLDER_TYPE_NORMAL:
+          {
+            wxTreeItemId id = data->GetId ();
+            wxTreeItemId parentId = treeCtrl->GetItemParent (id);
+            data = GetItemData (parentId);
+          }
+          break;
+
+        case FOLDER_TYPE_BOOKMARK:
+          context = bookmarks.GetContext (data->getPath ());
+          ok = context != 0;
+
+          break;
+        default:
+          ok = false;
+          break;
         }
-        break;
 
-      case FOLDER_TYPE_BOOKMARK:
-        context = bookmarks.GetContext (data->getPath ());
-        ok = context != 0;
-
-        break;
-      default:
-        ok = false;
-        break;
+        if (!ok)
+          break;
       }
-
-      if (!ok)
-        break;
     }
 
     if (context != 0)
@@ -248,7 +251,7 @@ public:
 
 
   void
-  ShowMenu (long index, wxPoint & pt)
+  ShowMenu (wxPoint & pt)
   {
     const FolderItemData * data = GetSelection ();
     if (!data)
@@ -260,7 +263,10 @@ public:
     int type =  data->getFolderType ();
 
     AppendMenuItem (menu, ID_AddWcBookmark);
+    AppendMenuItem (menu, ID_Checkout);
+    menu.AppendSeparator ();
     AppendMenuItem (menu, ID_AddRepoBookmark);
+    AppendMenuItem (menu, ID_CreateRepository);
 
     if (type==FOLDER_TYPE_BOOKMARK)
     {
@@ -566,23 +572,23 @@ public:
       if (!data->isReal ())
         break;
 
-      const wxString nodePath (data->getPath ());
-      if (nodePath.Length () == 0)
+      const svn::Path nodePath (data->getPath ());
+      if (nodePath.length () == 0)
         break;
 
       // first check: full match?
-      if (nodePath == path)
+      if (path.IsSameAs (nodePath.c_str()))
       {
         childId = id;
         break;
       }
 
       // second check: match until path delimiter
-      wxString prefix (path.Left (nodePath.Length ()));
-      wxString sep = path.Mid (nodePath.Length (), 1);
+      wxString prefix (path.Left (nodePath.length ()));
+      wxString sep = path.Mid (nodePath.length (), 1);
 
-      if ( (prefix == nodePath) && 
-           IsValidSeparator (sep) )
+      if (prefix.IsSameAs (nodePath.c_str()) &&
+           IsValidSeparator (sep))
       {
         childId = id;
         break;
@@ -629,25 +635,25 @@ public:
       if (data == 0)
         break;
 
-      wxString nodePath (data->getPath ());
-      if (nodePath.Length () == 0)
-        break;
-
       if (!data->isReal ())
         break;
 
+      svn::Path nodePath (data->getPath ());
+      if (nodePath.length () == 0)
+       break;
+
       // check if @a path and @a nodePath match already
       // in this case we are done
-      if (path == nodePath)
+      if (path.IsSameAs (nodePath.c_str()))
       {
         success = true;
         break;
       }
 
-      wxString prefix (path.Left (nodePath.Length ()));
-      wxString sep (path.Mid (nodePath.Length (), 1));
-      if ((prefix != nodePath) || !IsValidSeparator (sep))
-        break;
+      wxString prefix (path.Left (nodePath.length ()));
+      wxString sep (path.Mid (nodePath.length (), 1));
+      if ((!prefix.IsSameAs (nodePath.c_str())) || !IsValidSeparator (sep))
+       break;
 
       if (!data->hasChildren ())
         break;
@@ -767,8 +773,9 @@ public:
 BEGIN_EVENT_TABLE (FolderBrowser, wxControl)
   EVT_TREE_ITEM_EXPANDING (-1, FolderBrowser::OnExpandItem)
   EVT_TREE_ITEM_COLLAPSED (-1, FolderBrowser::OnCollapseItem)
-  EVT_TREE_ITEM_RIGHT_CLICK (-1, FolderBrowser::OnItemRightClk)
   EVT_SIZE (FolderBrowser::OnSize) 
+  EVT_TREE_ITEM_RIGHT_CLICK (-1, FolderBrowser::OnItemRightClk)
+  EVT_CONTEXT_MENU (FolderBrowser::OnContextMenu)
 END_EVENT_TABLE () 
 
 FolderBrowser::FolderBrowser (wxWindow * parent, const wxWindowID id,
@@ -876,15 +883,31 @@ FolderBrowser::OnCollapseItem (wxTreeEvent & event)
 void
 FolderBrowser::OnItemRightClk (wxTreeEvent & event)
 {
-  int flag;
-  wxPoint screenPt = wxGetMousePosition ();
-  wxPoint clientPt = ScreenToClient (screenPt);
+  RapidSvnFrame* frame = (RapidSvnFrame*) wxGetApp ().GetTopWindow ();
+  frame->SetActivePane (ACTIVEPANE_FOLDER_BROWSER);
 
-  long index = m->treeCtrl->HitTest (clientPt, flag);
-  if (index >= 0)
+#ifdef __WXGTK__
+  // wxGTK doesn't seem to select an item upon right clicking - try doing it ourselves for now
+  wxPoint clientPt = event.GetPoint ();
+  const wxTreeItemId id = m->treeCtrl->HitTest (clientPt);
+  if (id.IsOk ())
   {
-    m->ShowMenu (index, clientPt);
+    m->treeCtrl->SelectItem (id);
   }
+  // Show the menu now rather than waiting for OnContextMenu,
+  // and so don't Skip().
+  m->ShowMenu (clientPt);
+#else
+  // Let the OnContextMenu handler do the menu on MouseUp
+  event.Skip ();
+#endif
+}
+
+void 
+FolderBrowser::OnContextMenu (wxContextMenuEvent & event)
+{
+  wxPoint clientPt = ScreenToClient (event.GetPosition ());
+  m->ShowMenu (clientPt);
 }
 
 bool
