@@ -24,11 +24,18 @@
 #include "svncpp/wc.hpp"
 
 // app
+#include "config.hpp"
 #include "diff_data.hpp"
 #include "ids.hpp"
 #include "file_info.hpp"
 #include "listener.hpp"
 
+#ifdef USE_SIMPLE_WORKER
+#include "simple_worker.hpp"
+#else
+#include "threaded_worker.hpp"
+#endif
+
 // actions
 #include "add_action.hpp"
 #include "checkout_action.hpp"
@@ -73,6 +80,7 @@
 #include "res/bitmaps/resolve.xpm"
 #include "res/bitmaps/log.xpm"
 #include "res/bitmaps/info.xpm"
+#include "res/bitmaps/stop.xpm"
 
 // number of items initially in the list
 static const int NUM_ITEMS = 30;
@@ -82,6 +90,7 @@ const static char ConfigLeft[] = "/MainFrame/Left";
 const static char ConfigTop[] = "/MainFrame/Top";
 const static char ConfigWidth[] = "/MainFrame/Width";
 const static char ConfigHeight[] = "/MainFrame/Height";
+const static char ConfigMaximized[] = "/MainFrame/Maximized";
 const static char ConfigSplitterHoriz[] = "/MainFrame/SplitterHoriz";
 const static char ConfigSplitterVert[] = "/MainFrame/SplitterVert";
 const static char ConfigBookmarkFmt[] = "/Bookmarks/Bookmark%ld";
@@ -156,6 +165,24 @@ COLUMN_CAPTIONS[FileListCtrl::COL_COUNT] =
   _("Conflict Work")
 };
 
+
+/**
+ * Local helper function to create the action worker
+ *
+ * @param parent
+ * @return new action worker instance
+ */
+ActionWorker *
+CreateActionWorker (wxWindow * parent)
+{
+#ifdef USE_SIMPLE_WORKER
+  return new SimpleWorker (parent);
+#else
+  return new ThreadedWorker (parent);
+#endif
+}
+
+
 /**
  * class that hide implementation specific data and methods from
  * the interface
@@ -168,21 +195,28 @@ public:
   Listener listener;
   bool updateAfterActivate;
   bool dontUpdateFilelist;
+private:
+  bool m_running;
+  size_t m_toolbar_rows;        // 1 or 2 only (toolbar rows)
+  wxFrame * m_parent;
+
+public:
 
   /** 
    * This instance of @a apr is used to initialize/terminate apr 
    */
   svn::Apr apr;
 
-  Data (wxWindow * parent)
+  Data (wxFrame * parent)
     : MenuColumns (0), MenuBar (0), listener (parent),
-      updateAfterActivate (false), dontUpdateFilelist (false)
+      updateAfterActivate (false), dontUpdateFilelist (false),
+      m_running (false), m_toolbar_rows (1), m_parent (parent)
   {
     InitializeMenu ();
   }
 
   void
-  RapidSvnFrame::Data::InitializeMenu ()
+  InitializeMenu ()
   {
     // File menu
     wxMenu *menuFile = new wxMenu;
@@ -206,6 +240,7 @@ public:
     // View menu
     wxMenu *menuView = new wxMenu;
     AppendMenuItem (*menuView, ID_Refresh);
+    AppendMenuItem (*menuView, ID_Stop);
     menuView->AppendSeparator ();
 
     AppendMenuItem (*menuView, ID_Explore);
@@ -223,6 +258,7 @@ public:
     wxMenu *menuRepos = new wxMenu;
     menuRepos->Append (ID_Import, _("&Import...\tCTRL-I"));
     menuRepos->Append (ID_Checkout, _("Check&out...\tCTRL-O"));
+    menuRepos->Append (ID_CreateRepository, _("&Create..."));
 
     menuRepos->AppendSeparator ();
 
@@ -288,10 +324,175 @@ public:
     MenuBar->Check (id, check);
   }
 
+  void
+  SetRunning (bool running)
+  {
+    if (running && ! m_running)
+      listener.cancel (false);
+
+    m_running = running;
+
+    wxToolBarBase * toolBar = m_parent->GetToolBar ();
+
+    wxASSERT (toolBar != 0);
+
+    toolBar->EnableTool (ID_Stop, running);
+    MenuBar->Enable (ID_Stop, running);
+  }
+
+  bool 
+  IsRunning () const
+  {
+    return m_running;
+  }
+
+
+  void
+  RecreateToolbar ()
+  {
+    // delete the old toolbar
+    wxToolBarBase * toolBar = m_parent->GetToolBar ();
+    delete toolBar;
+
+    m_parent->SetToolBar (NULL);
+
+    long style = wxNO_BORDER | wxTB_FLAT | wxTB_DOCKABLE;
+    style |= wxTB_HORIZONTAL;
+
+    toolBar = m_parent->CreateToolBar (style, ID_TOOLBAR);
+    toolBar->SetMargins (4, 4);
+
+    AddActionTools (toolBar);
+    AddInfoTools (toolBar);
+
+    // Set toolbar refresh button.
+    toolBar->AddTool (ID_Refresh,
+                      wxBitmap (refresh_xpm),
+                      wxNullBitmap,
+                      FALSE,
+                      -1,
+                      -1,
+                      (wxObject *) NULL, 
+                      _("Refresh"), 
+                      _("Refresh the file list"));
+
+    toolBar->AddSeparator ();
+
+    // STOP button
+    toolBar->AddTool (ID_Stop,
+                      wxBitmap (stop_xpm),
+                      wxNullBitmap,
+                      FALSE,
+                      -1,
+                      -1,
+                      0,
+                      _("Stop"),
+                      _("Stop the current action"));
+
+    // After adding the buttons to the toolbar, 
+    // must call Realize() to reflect the changes.  
+    toolBar->Realize ();
+
+    toolBar->SetRows (m_toolbar_rows);
+  }
+
+  void
+  AddActionTools (wxToolBarBase *toolBar)
+  {
+    wxASSERT (toolBar);
+
+    toolBar->AddTool (ID_Add,
+                      wxBitmap (add_xpm),
+                      wxNullBitmap,
+                      FALSE,
+                      -1,
+                      -1,
+                      (wxObject *) NULL,
+                      _("Add selected"),
+                      _("Put files and directories under revision control"));
+
+    toolBar->AddTool (ID_Delete,
+                      wxBitmap (delete_xpm),
+                      wxNullBitmap,
+                      FALSE,
+                      -1,
+                      -1,
+                      (wxObject *) NULL,
+                      _("Delete selected"),
+                      _("Delete files and directories from version control"));
+
+    toolBar->AddTool (ID_Update,
+                      wxBitmap (update_xpm),
+                      wxNullBitmap,
+                      FALSE,
+                      -1,
+                      -1,
+                      (wxObject *) NULL,
+                      _("Update selected"),
+                      _("Bring changes from the repository into the working copy"));
+
+    toolBar->AddTool (ID_Commit,
+                      wxBitmap (commit_xpm),
+                      wxNullBitmap,
+                      FALSE,
+                      -1,
+                      -1,
+                      (wxObject *) NULL,
+                      _("Commit selected"),
+                      _("Send changes from your working copy to the repository"));
+
+    toolBar->AddTool (ID_Revert,
+                      wxBitmap (revert_xpm),
+                      wxNullBitmap,
+                      FALSE,
+                      -1,
+                      -1,
+                      (wxObject *) NULL,
+                      _("Revert selected"),
+                      _("Restore pristine working copy file (undo all local edits)"));
+
+    toolBar->AddTool (ID_Resolve,
+                      wxBitmap (resolve_xpm),
+                      wxNullBitmap,
+                      FALSE,
+                      -1,
+                      -1,
+                      (wxObject *) NULL,
+                      _("Resolve selected"),
+                      _("Remove 'conflicted' state on working copy files or directories)"));
+
+    toolBar->AddSeparator ();
+  }
+
+
+  void
+  AddInfoTools (wxToolBarBase *toolBar)
+  {
+    toolBar->AddTool (ID_Info,
+                      wxBitmap (info_xpm),
+                      wxNullBitmap,
+                      FALSE,
+                      -1,
+                      -1,
+                      (wxObject *) NULL,
+                      _("Info selected"),
+                      _("Display info about selected entries"));
+
+    toolBar->AddTool (ID_Log,
+                      wxBitmap (log_xpm),
+                      wxNullBitmap,
+                      FALSE,
+                      -1,
+                      -1,
+                      (wxObject *) NULL,
+                      _("Log selected"),
+                      _("Show the log messages for a set entries"));
+
+    toolBar->AddSeparator ();
+  }
 };
 
 BEGIN_EVENT_TABLE (RapidSvnFrame, wxFrame)
-  EVT_SIZE (RapidSvnFrame::OnSize)
   EVT_ACTIVATE (RapidSvnFrame::OnActivate)
   EVT_MENU (ID_AddWcBookmark, RapidSvnFrame::OnAddWcBookmark)
   EVT_MENU (ID_AddRepoBookmark, RapidSvnFrame::OnAddRepoBookmark)
@@ -310,6 +511,7 @@ BEGIN_EVENT_TABLE (RapidSvnFrame, wxFrame)
   EVT_MENU (ID_RefreshWithUpdate, RapidSvnFrame::OnRefreshWithUpdate)
   EVT_MENU (ID_Login, RapidSvnFrame::OnLogin)
   EVT_MENU (ID_Logout, RapidSvnFrame::OnLogout)
+  EVT_MENU (ID_Stop, RapidSvnFrame::OnStop)
 
   EVT_MENU_RANGE (ID_File_Min, ID_File_Max, RapidSvnFrame::OnFileCommand)
   EVT_MENU_RANGE (ID_Verb_Min, ID_Verb_Max, RapidSvnFrame::OnFileCommand)
@@ -327,15 +529,15 @@ BEGIN_EVENT_TABLE (RapidSvnFrame, wxFrame)
 END_EVENT_TABLE ()
 
 /** class implementation **/
-  RapidSvnFrame::RapidSvnFrame (const wxString & title)
-    : wxFrame ((wxFrame *) NULL, -1, title, wxDefaultPosition, wxDefaultSize, 
-               wxDEFAULT_FRAME_STYLE)
+RapidSvnFrame::RapidSvnFrame (const wxString & title)
+  : wxFrame ((wxFrame *) NULL, -1, title, wxDefaultPosition, wxDefaultSize, 
+             wxDEFAULT_FRAME_STYLE)
 {
   m = new Data (this);
   m_folder_browser = NULL;
   m_listCtrl = NULL;
   m_title = title;
-  m_actionWorker = new SimpleWorker (this);
+  m_actionWorker = CreateActionWorker (this);
   m_context = 0;
   m_activePane = ACTIVEPANE_FOLDER_BROWSER;
 
@@ -349,9 +551,6 @@ END_EVENT_TABLE ()
 
   SetIcon (wxIcon (svn_xpm));
 
-  // Toolbar
-  m_tbar = NULL;
-
   // toolbar rows
   m_toolbar_rows = 1;
 
@@ -359,7 +558,8 @@ END_EVENT_TABLE ()
   CreateStatusBar ();
 
   // Create the toolbar
-  RecreateToolbar ();
+  m->RecreateToolbar ();
+  m->SetRunning (false);
 
   m_horiz_splitter = 
     new ProportionalSplitterWindow (1.0f, this,
@@ -403,6 +603,7 @@ END_EVENT_TABLE ()
     m_folder_browser->SetListener (&m->listener);
     m_folder_browser->SetAuthPerBookmark (prefs.authPerBookmark);
   }
+  UpdateCurrentPath ();
 
   // Adapt the menu entries
   for (int col=0; col < FileListCtrl::COL_COUNT; col++)
@@ -423,15 +624,22 @@ END_EVENT_TABLE ()
   m_info_panel->SetSizer (sizer);
 
   // Read frame position
-  int x = pConfig->Read (ConfigLeft, 50);
-  int y = pConfig->Read (ConfigTop, 50);
-  int w = pConfig->Read (ConfigWidth, 806);
-  int h = pConfig->Read (ConfigHeight, 480);
+  if (pConfig->Read (ConfigMaximized, (long int)0) == 1)
+    Maximize (true);
+  else
+  {
+    int x = pConfig->Read (ConfigLeft, 50);
+    int y = pConfig->Read (ConfigTop, 50);
+    int w = pConfig->Read (ConfigWidth, 806);
+    int h = pConfig->Read (ConfigHeight, 480);
 
-  Move (x, y);
-  SetClientSize (w, h);
+    Move (x, y);
+    SetClientSize (w, h);
+  }
 
   // Get sash position for every splitter from configuration.
+  int w,h;
+  GetSize (&w, &h);
   int vpos = pConfig->Read (ConfigSplitterVert, w / 3);
   int hpos = pConfig->Read (ConfigSplitterHoriz, (3 * h) / 4);
 
@@ -457,17 +665,22 @@ RapidSvnFrame::~RapidSvnFrame ()
     delete m_actionWorker;
 
   // Save frame size and position.
+  if (IsMaximized ())
+    pConfig->Write (ConfigMaximized, 1);
+  else
+  {
+    int x, y;
+    int w, h;
 
-  int x, y;
-  int w, h;
+    GetClientSize (&w, &h);
+    GetPosition (&x, &y);
 
-  GetClientSize (&w, &h);
-  GetPosition (&x, &y);
-
-  pConfig->Write (ConfigLeft, (long) x);
-  pConfig->Write (ConfigTop, (long) y);
-  pConfig->Write (ConfigWidth, (long) w);
-  pConfig->Write (ConfigHeight, (long) h);
+    pConfig->Write (ConfigMaximized, 0);
+    pConfig->Write (ConfigLeft, (long) x);
+    pConfig->Write (ConfigTop, (long) y);
+    pConfig->Write (ConfigWidth, (long) w);
+    pConfig->Write (ConfigHeight, (long) h);
+  }
 
 
   // Save splitter positions
@@ -500,16 +713,21 @@ RapidSvnFrame::UpdateFileList ()
   if (m->dontUpdateFilelist)
     return;
 
-  wxBusyCursor busy;
+  m->SetRunning (true);
+
   if (m_listCtrl && m_folder_browser)
   {
-    if (m_currentPath.length () > 0)
+    wxBusyCursor busy;
+    if (m_currentPath.length () == 0)
+      m_listCtrl->Show (FALSE);
+    else
     {
       try
       {
         m_listCtrl->SetContext (m_context);
         m_listCtrl->UpdateFileList (m_currentPath);
         m_listCtrl->Show (TRUE);
+
       }
       catch (svn::ClientException & e)
       {
@@ -526,11 +744,9 @@ RapidSvnFrame::UpdateFileList ()
         Trace (_("Error while updating filelist"));
       }
     }
-    else
-    {
-      m_listCtrl->Show (FALSE);
-    }
   }
+
+  m->SetRunning (false);
 }
 
 void
@@ -589,38 +805,6 @@ RapidSvnFrame::OnRefresh (wxCommandEvent & WXUNUSED (event))
   UpdateFolderBrowser ();
 }
 
-void
-RapidSvnFrame::LayoutChildren ()
-{
-  wxSize size = GetClientSize ();
-
-  int offset;
-  if (m_tbar)
-  {
-    m_tbar->SetSize (-1, size.y);
-    m_tbar->Move (0, 0);
-
-    offset = m_tbar->GetSize ().x;
-  }
-  else
-  {
-    offset = 0;
-  }
-}
-
-void
-RapidSvnFrame::OnSize (wxSizeEvent & event)
-{
-  if (m_tbar)
-  {
-    LayoutChildren ();
-  }
-  else
-  {
-    event.Skip ();
-  }
-}
-
 
 void 
 RapidSvnFrame::OnActivate (wxActivateEvent & event)
@@ -634,145 +818,12 @@ RapidSvnFrame::OnActivate (wxActivateEvent & event)
 }
 
 
-void
-RapidSvnFrame::RecreateToolbar ()
-{
-  // delete the old toolbar
-  wxToolBarBase *toolBar = GetToolBar ();
-  delete toolBar;
-
-  SetToolBar (NULL);
-
-  long style = wxNO_BORDER | wxTB_FLAT | wxTB_DOCKABLE;
-  style |= wxTB_HORIZONTAL;
-
-  toolBar = CreateToolBar (style, ID_TOOLBAR);
-  toolBar->SetMargins (4, 4);
-
-  AddActionTools ();
-  AddInfoTools ();
-
-  // Set toolbar refresh button.
-  toolBar->AddTool (ID_Refresh,
-                    wxBitmap (refresh_xpm),
-                    wxNullBitmap,
-                    FALSE,
-                    -1,
-                    -1,
-                    (wxObject *) NULL, 
-                    _("Refresh"), 
-                    _("Refresh the file list"));
-
-  toolBar->AddSeparator ();
-
-  // After adding the buttons to the toolbar, 
-  // must call Realize() to reflect the changes.  
-  toolBar->Realize ();
-
-  toolBar->SetRows (m_toolbar_rows);
-}
-
-void
-RapidSvnFrame::AddActionTools ()
-{
-  wxToolBarBase *toolBar = GetToolBar ();
-
-  toolBar->AddTool (ID_Add,
-                    wxBitmap (add_xpm),
-                    wxNullBitmap,
-                    FALSE,
-                    -1,
-                    -1,
-                    (wxObject *) NULL,
-                    _("Add selected"),
-                    _("Put files and directories under revision control"));
-
-  toolBar->AddTool (ID_Delete,
-                    wxBitmap (delete_xpm),
-                    wxNullBitmap,
-                    FALSE,
-                    -1,
-                    -1,
-                    (wxObject *) NULL,
-                    _("Delete selected"),
-                    _("Delete files and directories from version control"));
-
-  toolBar->AddTool (ID_Update,
-                    wxBitmap (update_xpm),
-                    wxNullBitmap,
-                    FALSE,
-                    -1,
-                    -1,
-                    (wxObject *) NULL,
-                    _("Update selected"),
-                    _("Bring changes from the repository into the working copy"));
-
-  toolBar->AddTool (ID_Commit,
-                    wxBitmap (commit_xpm),
-                    wxNullBitmap,
-                    FALSE,
-                    -1,
-                    -1,
-                    (wxObject *) NULL,
-                    _("Commit selected"),
-                    _("Send changes from your working copy to the repository"));
-
-  toolBar->AddTool (ID_Revert,
-                    wxBitmap (revert_xpm),
-                    wxNullBitmap,
-                    FALSE,
-                    -1,
-                    -1,
-                    (wxObject *) NULL,
-                    _("Revert selected"),
-                    _("Restore pristine working copy file (undo all local edits)"));
-
-  toolBar->AddTool (ID_Resolve,
-                    wxBitmap (resolve_xpm),
-                    wxNullBitmap,
-                    FALSE,
-                    -1,
-                    -1,
-                    (wxObject *) NULL,
-                    _("Resolve selected"),
-                    _("Remove 'conflicted' state on working copy files or directories)"));
-
-  toolBar->AddSeparator ();
-}
-
-void
-RapidSvnFrame::AddInfoTools ()
-{
-  wxToolBarBase *toolBar = GetToolBar ();
-
-  toolBar->AddTool (ID_Info,
-                    wxBitmap (info_xpm),
-                    wxNullBitmap,
-                    FALSE,
-                    -1,
-                    -1,
-                    (wxObject *) NULL,
-                    _("Info selected"),
-                    _("Display info about selected entries"));
-
-  toolBar->AddTool (ID_Log,
-                    wxBitmap (log_xpm),
-                    wxNullBitmap,
-                    FALSE,
-                    -1,
-                    -1,
-                    (wxObject *) NULL,
-                    _("Log selected"),
-                    _("Show the log messages for a set entries"));
-
-  toolBar->AddSeparator ();
-}
-
 void
 RapidSvnFrame::OnToolEnter (wxCommandEvent & event)
 {
 }
 
+
 void
 RapidSvnFrame::AddWcBookmark ()
 {
@@ -1038,7 +1089,11 @@ RapidSvnFrame::ValidateIDActionFlags (int id, unsigned int selectionActionFlags)
       // Not actually part of the Action hierarchy, but here for completeness
       baseActionFlags = Action::SINGLE_TARGET|Action::MULTIPLE_TARGETS|Action::RESPOSITORY_TYPE|Action::VERSIONED_WC_TYPE|Action::UNVERSIONED_WC_TYPE;
       break;
-  
+
+    case ID_Edit:
+      baseActionFlags = ViewAction::GetEditFlags ();
+      break;
+
     case ID_Contents: //TODO
     default:
       // If unrecognised, by default return true
@@ -1110,6 +1165,12 @@ RapidSvnFrame::TrimDisabledMenuItems (wxMenu & menu)
   }
 }
 
+void
+RapidSvnFrame::SetActivePane (ActivePane value)
+{
+  m_activePane = value;
+}
+
 void
 RapidSvnFrame::OnFileCommand (wxCommandEvent & event)
 {
@@ -1207,6 +1268,23 @@ RapidSvnFrame::OnFileCommand (wxCommandEvent & event)
       action = new DiffAction (this);
       break;
 
+    case ID_Edit:
+      action = new ViewAction (this);
+      break;
+
+    case ID_CreateRepository:
+      wxMessageBox (
+        _("Please use the command line utility 'svnadmin'\n"
+          "to create a new repository.\n\n"
+          "This command line utility is not part of the\n"
+          "RapidSVN distribution.\n\n"
+          "More information about subversion:\n"
+          "http://svnbook.red-bean.com/\n"
+          "http://subversion.tigris.org"),
+        _("Information"),
+        wxOK);
+      break;
+
     case ID_Contents: //TODO
     default:
       m_logTracer->Trace ("Unimplemented action!");
@@ -1234,6 +1312,7 @@ RapidSvnFrame::OnActionEvent (wxCommandEvent & event)
     Trace (event.GetString ());
     UpdateFileList ();
     Trace (_("Ready\n"));
+    m->SetRunning (false);
     break;
 
   case TOKEN_ACTION_START:
@@ -1243,8 +1322,17 @@ RapidSvnFrame::OnActionEvent (wxCommandEvent & event)
 
   case TOKEN_ACTION_END:
     {
-      unsigned int actionFlags = 
-        (unsigned int)event.GetClientData ();
+      unsigned int* af = (unsigned int*) event.GetClientData ();
+      unsigned int actionFlags = 0;
+      if (af)
+      {
+        actionFlags = *af;
+        delete af;
+      }
+      else
+      {
+        Trace (_("Error receiving action event!"));
+      }
 
       if ((actionFlags & Action::UPDATE_LATER) != 0)
       {
@@ -1260,6 +1348,7 @@ RapidSvnFrame::OnActionEvent (wxCommandEvent & event)
       }
 
       Trace (_("Ready\n"));
+      m->SetRunning (false);
     }
     break;
 
@@ -1327,6 +1416,31 @@ RapidSvnFrame::OnActionEvent (wxCommandEvent & event)
     }
     break;
 
+  case TOKEN_MERGE:
+    {
+      MergeData * pData = static_cast<MergeData *>(event.GetClientData ());
+
+      if (pData != 0)
+      {
+        // copy and delete data
+        MergeData data (*pData);
+        delete pData;
+        Action * action;
+
+        action = new MergeAction (this, data);
+        Perform (action);
+      }
+    }
+    break;
+
+  case TOKEN_DELETE_ACTION:
+    {
+      Action * action = static_cast<Action *>(event.GetClientData ());
+
+      if (action != 0)
+        delete action;
+    }
+    break;
   }
 }
 
@@ -1442,16 +1556,28 @@ RapidSvnFrame::InvokeDefaultAction ()
 void
 RapidSvnFrame::Perform (Action * action)
 {
-  action->SetPath (m_currentPath.c_str ());
-  action->SetContext (m_context);
-  if ((action->GetFlags () & Action::WITHOUT_TARGET) == 0)
+  try
   {
-    action->SetTargets (GetActionTargets ());
+    m->SetRunning (true);
+    m->listener.cancel (false);
+
+    action->SetPath (m_currentPath.c_str ());
+    action->SetContext (m_context);
+    if ((action->GetFlags () & Action::WITHOUT_TARGET) == 0)
+    {
+      action->SetTargets (GetActionTargets ());
+    }
+    action->SetTracer (m_logTracer, false);
+    m_actionWorker->SetTracer (m_logTracer);
+    m_actionWorker->SetContext (m_context, false);
+    if (!m_actionWorker->Perform (action))
+      m->SetRunning (false);
+  }
+  catch (...)
+  {
+    m->SetRunning (false);
+    throw; // svn::Excepton (e);
   }
-  action->SetTracer (m_logTracer, false);
-  m_actionWorker->SetTracer (m_logTracer);
-  m_actionWorker->SetContext (m_context, false);
-  m_actionWorker->Perform (action);
 }
 
 void 
@@ -1518,6 +1644,8 @@ RapidSvnFrame::UpdateFolderBrowser ()
   m_currentPath = "";
   UpdateFileList ();
 
+  m->SetRunning (true);
+
   try
   {
     m->dontUpdateFilelist = true;
@@ -1529,6 +1657,8 @@ RapidSvnFrame::UpdateFolderBrowser ()
   {
   }
 
+  m->SetRunning (false);
+
   m->dontUpdateFilelist = false;
   UpdateFileList ();
 }
@@ -1572,6 +1702,12 @@ RapidSvnFrame::OnLogout (wxCommandEvent & event)
   context->setLogin ("", "");
 }
 
+void 
+RapidSvnFrame::OnStop (wxCommandEvent & event)
+{
+  m->listener.cancel (true);
+}
+
 InfoPanel::InfoPanel (wxWindow * parent)
   : wxPanel (parent, -1, wxDefaultPosition, wxDefaultSize, 
              wxTAB_TRAVERSAL | wxCLIP_CHILDREN)
