@@ -38,7 +38,7 @@ ResolveAction::Perform ()
   const std::vector<svn::Path> v = GetTargets ();
   std::vector<svn::Path>::const_iterator it;
 
-  svn::Client client;
+  svn::Client client (GetContext ());
   for (it = v.begin (); it != v.end (); it++)
   {
     const svn::Path & path = *it;
