@@ -31,6 +31,7 @@
 #include "ids.hpp"
 #include "utils.hpp"
 #include "tracer.hpp"
+#include "verblist.hpp"
 
 // bitmaps
 #include "res/bitmaps/add.xpm"
@@ -136,6 +137,9 @@ CreateEllipsisButton(wxWindow *parent, long id)
 void 
 AppendModifyMenu (wxMenu * parentMenu)
 {
+  AppendMenuItem (*parentMenu, ID_Edit);
+
+  parentMenu->AppendSeparator ();
   AppendMenuItem (*parentMenu, ID_Update);
   AppendMenuItem (*parentMenu, ID_Commit);
 
@@ -210,13 +214,17 @@ AppendMenuItem (wxMenu & menu, int id)
 
   switch (id)
   {
+  case ID_Edit:
+    caption = _("&Edit...\tF3");
+    break;
+
   case ID_AddWcBookmark:
-    caption = _("&Add Working Copy...");
+    caption = _("&Add Existing Working Copy...");
     bitmap = wxBitmap (add_wc_bookmark_xpm);
     break;
 
   case ID_AddRepoBookmark:
-    caption = _("Add &Repository...");
+    caption = _("Add Existing &Repository...");
     break;
   
   case ID_RemoveBookmark:
@@ -224,6 +232,14 @@ AppendMenuItem (wxMenu & menu, int id)
     bitmap = wxBitmap (remove_bookmark_xpm);
     break;
 
+  case ID_CreateRepository:
+    caption = _("Create New Repository...");
+    break;
+
+  case ID_Checkout:
+    caption = _("Checkout New Working Copy...");
+    break;
+
   case ID_Login:
     caption =  _("Login...");
     break;
@@ -257,6 +273,11 @@ AppendMenuItem (wxMenu & menu, int id)
   case ID_Column_Reset:
     caption = _("Reset Columns");
     break;
+
+  case ID_Stop:
+    caption = _("&Stop");
+    break;
+
   case ID_Explore:
     caption = _("Explore...\tF2");
   }
@@ -390,6 +411,65 @@ StatusDescription (const svn_wc_status_kind kind)
 }
 
 
+wxString
+Utf8ToLocal (const wxString & srcUtf8)
+{
+  wxString dst (srcUtf8, wxConvUTF8);
+
+  return srcUtf8;
+}
+
+wxString
+LocalToUtf8 (const wxString & srcLocal)
+{
+  wxString dst (srcLocal.mb_str (wxConvUTF8));
+
+  return dst;
+}
+
+
+void
+AppendVerbMenu (wxMenu * parentMenu, svn::Status * status)
+{
+    wxASSERT (status);
+
+    // Append file verbs
+    try
+    {
+      VerbList verbList;
+
+      verbList.InitFromDocument (status->path ());
+
+      if (verbList.GetCount () == 0)
+        return;
+
+      wxMenu * menu = new wxMenu ();
+
+      size_t i = 0;
+      for (; 
+           (i < verbList.GetCount ()) && 
+           (i < (ID_Verb_Max - ID_Verb_Min + 1)); i++)
+      {
+        wxMenuItem *pItem;
+        // TODO: Convert verb names to unicode on the fly if needed (or make
+        // verblist follow wxWindows' unicode setting)
+        pItem = new wxMenuItem (menu, ID_Verb_Min + i, verbList.GetName (i));
+        //pItem->SetBitmap (wxBITMAP (?))
+        menu->Append (pItem);
+      }
+    
+      parentMenu->Append (ID_Open, _("Open..."), menu);
+    }
+    catch (std::exception)
+    {
+      // Failed assembling verbs. 
+      // TODO: Report this error in the status bar?
+
+    }
+
+}
+
+
 
 /* -----------------------------------------------------------------
  * local variables:
