@@ -26,6 +26,11 @@ class Tracer;
 class wxMenu;
 class wxCommandEvent;
 
+namespace svn
+{
+  class Status;
+}
+
 
 /**
  * declaration of an array with @a svn_revnum_t elements
@@ -95,6 +100,20 @@ AppendBookmarksMenu (wxMenu * parentMenu);
 void 
 AppendQueryMenu (wxMenu * parentMenu);
 
+
+/**
+ * Append entries for "verbs" (Win32 only).
+ * Since this feature is context specific (based on
+ * the selected file), @a status has to be passed.
+ * Works only with one selected file
+ *
+ * @param parentMenu menu that will receive the items
+ * @param status status instance of the selected item
+ */
+void
+AppendVerbMenu (wxMenu * parentMenu, svn::Status * status);
+
+
 /**
  * Checks if the given string contains a valid
  * revision number
@@ -207,6 +226,32 @@ wxString
 StatusDescription (const svn_wc_status_kind kind);
 
 
+/**
+ * converts a string from local encoding (like Ansi on 
+ * Windows) to utf8
+ *
+ * @see Utf8ToLocal
+ *
+ * @param srcLocal string in local encoding
+ * @return string in local encoding
+ */
+wxString
+LocalToUtf8 (const wxString & srcLocal);
+
+
+/**
+ * converts a string from utf to the local encoding 
+ * (like Ansi on Windows)
+ *
+ * @see LocalToUtf8
+ *
+ * @param srcUtf8 string in utf8 encoding
+ * @return string in utf8 encoding
+ */
+wxString
+Utf8ToLocal (const wxString & srcUtf8);
+
+
 #endif
 /* -----------------------------------------------------------------
  * local variables:
